<?php
// $Id: hellomobile-admin.inc,v 1.4 2009/05/19 03:30:22 teemule Exp $

/**
 * Creates the settings form.
 * @return the settings form
 */
function hellomobile_admin_settings() {
	$form = array();
	$form['domains'] = array(
		'#type' => 'fieldset',
		'#title' => t('Domain based mobile version'),
		'#description' => t('Optional domains used to select the site version. Don\'t use http://')
	);
	$form['domains']['hellomobile_mobile_domain'] = array(
		'#type' => 'textfield',
		'#title' => t('Domain for mobile site'),
		'#default_value' => variable_get('hellomobile_mobile_domain', ''),
	);
	$form['domains']['hellomobile_full_domain'] = array(
		'#type' => 'textfield',
		'#title' => t('Domain for full site'),
		'#default_value' => variable_get('hellomobile_full_domain', ''),
	);
	$form['wurfl'] = array(
		'#type' => 'fieldset',
		'#title' => t('Automatic device detection'),
		'#description' => l(t('Test WURFL detection'), 'hellomobile/wurfldebug')
	);
	$form['wurfl']['hellomobile_detection'] = array(
		'#type' => 'checkbox',
		'#title' => t('Use automatic device detection (WURFL)'),
		'#default_value' => variable_get('hellomobile_detection', true)
	);
	$form['tools'] = array(
		'#type' => 'fieldset',
		'#title' => t('Administrative tools')
	);
	$form['tools']['flush'] = array(
		'#type' => 'button',
		'#value' => t('Flush cache and scaled images')
	);
	if (variable_get('cache', 0)) {
		drupal_set_message(l(t('Disable page cache or multiple themes will not work.'), 'admin/settings/performance'), 'error');
	}
	if (!variable_get('preprocess_css', false)) {
		drupal_set_message(l(t('Using aggregate and compress CSS files is recommended.'), 'admin/settings/performance'), 'error');
	}
	return system_settings_form($form);
}

/**
 * Validates the settings form.
 * @param form_id an unique id
 * @param form_values the sent form values
 */
function hellomobile_admin_settings_validate($form_id, $form_values) {
	drupal_get_messages();
	if ($form_values['op'] == t('Flush cache and scaled images')) {
		_hellomobile_flush();
	}
	if (substr($form_values['hellomobile_mobile_domain'], 0, 7) == 'http://') {
		form_set_error('hellomobile_mobile_domain', t('Do not use http://'));
	}
	if (substr($form_values['hellomobile_full_domain'], 0, 7) == 'http://') {
		form_set_error('hellomobile_full_domain', t('Do not use http://'));
	}
}

/**
 * Lists device groups.
 * @return page content
 */
function hellomobile_groups() {
	$c = '<ul>';
	$result = db_query('select name,theme from {hellomobile_groups}');
	while($row = db_fetch_array($result)) {
		$c .= '<li><strong>' . check_plain($row['name']) . '</strong> (' . check_plain($row['theme']) . ')<br />'
			. l(t('Edit group'), 'admin/build/hellomobile/groups/edit/' . urlencode($row['name'])) . '<br />'
			. l(t('Filter group css and js'), 'admin/build/hellomobile/groups/filter/' . urlencode($row['name'])) . '<br />'
			. l(t('Edit group blocks'), 'admin/build/block/list/' . urlencode($row['theme'])) . '<br />'
			. l(t('Test group until the end of the session'), 'admin/build/hellomobile/test/' . urlencode($row['name'])) . '</li>';
	}
	$c .= '</ul><p>' . l(t('Add new group'), 'admin/build/hellomobile/groups/edit') . '</p>';
	$c .= '<p>' . l(t('Test touchscreen class until the end of the session'), 'admin/build/hellomobile/test/_touchscreen') . '</p>';
	if (isset($_SESSION['hellomobile_group'])) {
		$c .= '<p>' . l(t('Stop testing group') . ': ' . check_plain($_SESSION['hellomobile_group']), 'admin/build/hellomobile/test') . '</p>';
	}
	return $c;
}

/**
 * Creates the group edit form.
 * @param name a group name
 * @return the edit form
 */
function hellomobile_groups_form($name = false) {
	$form = array();
	if ($name) {
		$group = _hellomobile_get_group_setup($name);
		if (!$group) {
			drupal_not_found();
			exit();
		}
		_hellomobile_group_setup_in($group);
		$title = t('Edit group') . ': ' . $group['name'];
		$desc = t('Settings for a device group. Set up rules to pick devices.');
	} else {
		$group = array('name' => '', 'theme' => '', 'images' => true, 'youtube' => true);
		$title = t('Group settings');
		$desc = t('Create a new device group and settings for it.');
		$form['group_name'] = array(
			'#type' => 'textfield',
			'#required' => true,
			'#title' => t('Group name')
		);
	}
	$form['group'] = array(
		'#type' => 'fieldset',
		'#title' => $title,
		'#description' => $desc
	);
	$form['group']['name'] = array(
		'#type' => 'hidden',
		'#default_value' => $group['name']
	);
	$form['group']['theme'] = array(
		'#type' => 'select',
		'#title' => t('Drupal theme'),
		'#default_value' => _hellomobile_get_setup_val($group, 'theme', 'mobile_garland'),
		'#options' => _hellomobile_get_theme_names()
	);
	$form['group']['breakwords'] = array(
		'#type' => 'textfield',
		'#title' => t('Suggest a word break after this many characters, zero to disable'),
		'#default_value' => _hellomobile_get_setup_val($group, 'breakwords', 30)
	);
	$form['group']['imageact'] = array(
		'#type' => 'select',
		'#title' => t('Image action'),
		'#default_value' => _hellomobile_get_setup_val($group, 'imageact', 'scale'),
		'#options' => array('none' => t('Do nothing'), 'scale' => t('Scale down'), 'remove' => t('Remove'))
	);
	$form['group']['imagesize'] = array(
		'#type' => 'textfield',
		'#title' => t('Maximum image size as width x height'),
		'#default_value' => _hellomobile_get_setup_val($group, 'imagesize', '180x180')
	);
	$form = array_merge($form, module_invoke_all('hellomobile_setup', $group));
	$form['submit'] = array(
		'#type' => 'submit',
		'#value' => t('Save')
	);
	if ($name) {
		$form['delete'] = array(
			'#type' => 'submit',
			'#value' => t('Delete group')
		);
	}
	$form['cancel'] = array(
		'#type' => 'button',
		'#value' => t('Cancel')
	);
	return $form;
}

/**
 * Gets a group setup by group name.
 * @param group_name a group name
 * @return a group setup
 */
function _hellomobile_get_group_setup($group_name) {
	$result = db_query('select name,theme,data from {hellomobile_groups} where name="%s"', $group_name);
	return db_fetch_array($result);
}

/**
 * Gets enabled themes.
 * @return an array of enabled themes
 */
function _hellomobile_get_theme_names() {
	$themes = array();
	foreach (list_themes() as $t) {
		if ($t->status) {
			$themes[$t->name] = $t->name;
		}
	}
	return $themes;
}

/**
 * Validates the group edit form.
 * @param form_id an unique id
 * @param form_values the sent form values
 */
function hellomobile_groups_form_validate($form_id, $form_values) {
	if ($form_values['op'] == t('Cancel')) {
		drupal_get_messages();
		drupal_goto('admin/build/hellomobile/groups');
		exit();
	}
	if (isset($form_values['group_name'])) {
		if (db_result(db_query('select name from {hellomobile_groups} where name="%s"', $form_values['group_name']))) {
			form_set_error('group_name', t('Group with a same name exists.'));
		}
	}
}

/**
 * Submits the group edit form.
 * @param form_id an unique id
 * @param form_values the sent form values
 */
function hellomobile_groups_form_submit($form_id, $form_values) {
	$pick_keys = array('op', 'name', 'group_name', 'theme');
	$drop_keys = array('submit', 'delete', 'cancel', 'form_token', 'form_id');
	$pick = array();
	foreach ($pick_keys as $key) {
		$pick[$key] = $form_values[$key];
	}
	foreach (array_merge($pick_keys, $drop_keys) as $key) {
		unset($form_values[$key]);
	}
	if ($pick['op'] == t('Delete group')) {
		db_query('delete from {hellomobile_groups} where name="%s"', $pick['name']);
		drupal_set_message(t('The group deleted.'));
	} else if (strlen($pick['name']) > 0) {
		$group = _hellomobile_get_group_setup($pick['name']);
		if ($group) {
			_hellomobile_group_setup_in($group);
			$form_values['filtercss'] = $group['filtercss'];
			$form_values['filterjs'] = $group['filterjs'];
			db_query('update {hellomobile_groups} set theme="%s",data="%s" where name="%s"', $pick['theme'], serialize($form_values), $pick['name']);
			drupal_set_message(t('Changes to the group saved.'));
		}
	} else {
		db_query('insert into {hellomobile_groups} (name,theme,data) values ("%s","%s","%s")', $pick['group_name'], $pick['theme'], serialize($form_values));
		drupal_set_message(t('New group added.'));
	}
	cache_clear_all('hellomobile_grouplist', 'cache');
	cache_clear_all('hellomobile_optimized:*', 'cache_page', true);
	cache_clear_all();
	return 'admin/build/hellomobile/groups';
}

/**
 * Creates the group filter css and js form.
 * @param name a group name
 * @return the edit form
 */
function hellomobile_filter_form($name = false) {
	if (!$name) {
		drupal_not_found();
		exit();
	}
	$result = db_query('select name,theme,data from {hellomobile_groups} where name="%s"', $name);
	$group = db_fetch_array($result);
	if (!$group) {
		drupal_not_found();
		exit();
	}
	_hellomobile_group_setup_in($group);
	$form = array();
	$form['name'] = array(
		'#type' => 'hidden',
		'#value' => $group['name']
	);
	$form['css'] = array(
		'#type' => 'textarea',
		'#title' => t('Allow CSS files for group') . ' <em>' . $group['name'] . '</em>',
		'#default_value' => _hellomobile_list2textarea(_hellomobile_get_setup_val($group, 'filtercss', array('sites/all/themes'))),
		'#description' => t('List a path per line. Stylesheets from these locations are allowed.')
	);
	$form['js'] = array(
		'#type' => 'textarea',
		'#title' => t('Allow JS files for group') . ' <em>' . $group['name'] . '</em>',
		'#default_value' => _hellomobile_list2textarea(_hellomobile_get_setup_val($group, 'filterjs', array())),
		'#description' => t('List a path per line. Javascripts from these locations are allowed.')
	);
	$form['submit'] = array(
		'#type' => 'submit',
		'#value' => t('Save')
	);
	$form['cancel'] = array(
		'#type' => 'button',
		'#value' => t('Cancel')
	);
	return $form;
}

/**
 * Validates the group filter css and js form.
 * @param form_id an unique id
 * @param form_values the sent form values
 */
function hellomobile_filter_form_validate($form_id, $form_values) {
	if ($form_values['op'] == t('Cancel')) {
		drupal_get_messages();
		drupal_goto('admin/build/hellomobile/groups');
		exit();
	}
}

/**
 * Submits the group filter css and js form.
 * @param form_id an unique id
 * @param form_values the sent form values
 */
function hellomobile_filter_form_submit($form_id, $form_values) {
	$group = _hellomobile_get_group_setup($form_values['name']);
	if (!$group) {
		drupal_not_found();
		exit();
	}
	$data = array();
	if ($group['data']) {
		$data = unserialize($group['data']);
	}
	$data['filtercss'] = _hellomobile_textarea2list($form_values['css']);
	$data['filterjs'] = _hellomobile_textarea2list($form_values['js']);
	db_query('update {hellomobile_groups} set data="%s" where name="%s"', serialize($data), $group['name']);
	drupal_set_message(t('Changes to the css and js filter saved.'));
	cache_clear_all('hellomobile_grouplist', 'cache');
	cache_clear_all();
	return 'admin/build/hellomobile/groups';
}

/**
 * Gets an editable textarea content for a list.
 * @param list a list of values
 * @return each value on a line
 */
function _hellomobile_list2textarea($list) {
	$content = '';
	foreach ($list as $value) {
		$content .= $value . "\n";
	}
	return $content;
}

/**
 * Gets a list from edited textarea content.
 * @param content each value on a line
 * @return a list of values
 */
function _hellomobile_textarea2list($content) {
	$list = array();
	foreach (explode("\n", $content) as $value) {
		$value = trim($value);
		if (!empty($value)) {
			$list[] = $value;
		}
	}
	return $list;
}

/**
 * Lists detection rules.
 * @return page content
 */
function hellomobile_rules() {
	$c = '<ul>';
	$result = db_query('select rid,capability,operator,value,group_name,weight from {hellomobile_rules} order by weight asc');
	while($row = db_fetch_array($result)) {
		$c .= '<li>' . l(t('Edit rule:'), 'admin/build/hellomobile/rules/edit/' . intval($row['rid']))
			. ' <strong>' . check_plain($row['capability'] . ' ' . $row['operator'] . ' ' . $row['value'])
			. '</strong> ? group=<strong>' . $row['group_name'] . '</strong> (weight=' . intval($row['weight']) . ')</li>';
	}
	$c .= '</ul><p>' . l(t('Add new rule'), 'admin/build/hellomobile/rules/edit') . '</p>';
	return $c;
}

/**
 * Creates the rules edit form.
 * @param rid a rule id
 * @return the edit form
 */
function hellomobile_rules_form($rid = '') {
	$form = array();
	if ($rid != '') {
		$result = db_query('select rid,capability,operator,value,group_name,weight from {hellomobile_rules} where rid="%d"', $rid);
		$rule = db_fetch_array($result);
		if (!$rule) {
			drupal_not_found();
			exit();
		}
	} else {
		$rule = array('rid' => '', 'capability' => '', 'operator' => '==', 'value' => '', 'group_name' => '', 'weight' => 0);
	}
	$form['rule'] = array(
		'#type' => 'fieldset',
		'#title' => t('Edit rule'),
		'#description' => t('Devices will be assigned to a group by the lightest matching rule.')
	);
	$form['rule']['rid'] = array(
		'#type' => 'hidden',
		'#default_value' => $rule['rid']
	);
	$form['rule']['capability'] = array(
		'#type' => 'textfield',
		'#required' => true,
		'#title' => t('Capability'),
		'#default_value' => $rule['capability'],
		'#description' => t('WURFL capability name to inspect or "ID" for WURFL ID or "USERAGENT" for HTTP User-Agent field.')
	);
	$form['rule']['operator'] = array(
		'#type' => 'select',
		'#required' => true,
		'#title' => t('Operator'),
		'#default_value' => '==',
		'#options' => array('==' => '==', '!=', '!=', '<=' => '<=', '>=' => '>=', '=~' => '=~')
	);
	$form['rule']['value'] = array(
		'#type' => 'textfield',
		'#required' => true,
		'#title' => t('Value'),
		'#default_value' => $rule['value'],
		'#description' => t('Value to compare to.')
	);
	$form['rule']['group_name'] = array(
		'#type' => 'select',
		'#required' => true,
		'#title' => t('Group'),
		'#default_value' => $rule['group_name'],
		'#options' => _hellomobile_get_group_names(),
		'#description' => t('A group to assign when the rule matches')
	);
	$form['rule']['weight'] = array(
		'#type' => 'weight',
		'#title' => t('Weight'),
		'#default_value' => $rule['weight']
	);
	$form['submit'] = array(
		'#type' => 'submit',
		'#value' => t('Save')
	);
	if ($rid != '') {
		$form['delete'] = array(
			'#type' => 'submit',
			'#value' => t('Delete rule')
		);
	}
	$form['cancel'] = array(
		'#type' => 'button',
		'#value' => t('Cancel')
	);
	return $form;
}

/**
 * Gets Hello Mobile groups.
 * @return an array of Hello Mobile groups
 */
function _hellomobile_get_group_names() {
	$groups = array('no' => 'No detection');
	$result = db_query('select name from {hellomobile_groups}');
	while ($row = db_fetch_array($result)) {
		$groups[$row['name']] = $row['name'];
	}
	return $groups;
}

/**
 * Validates the group edit form.
 * @param form_id an unique id
 * @param form_values the sent form values
 */
function hellomobile_rules_form_validate($form_id, $form_values) {
	if ($form_values['op'] == t('Cancel')) {
		drupal_get_messages();
		drupal_goto('admin/build/hellomobile/rules');
		exit();
	}
}

/**
 * Submits the group edit form.
 * @param form_id an unique id
 * @param form_values the sent form values
 */
function hellomobile_rules_form_submit($form_id, $form_values) {
	if ($form_values['op'] == t('Delete rule')) {
		db_query('delete from {hellomobile_rules} where rid=%d', $form_values['rid']);
		drupal_set_message(t('The rule deleted.'));
	} else if (strlen($form_values['rid']) > 0) {
		db_query('update {hellomobile_rules} set capability="%s",operator="%s",value="%s",group_name="%s",weight=%d where rid=%d', $form_values['capability'], $form_values['operator'], $form_values['value'], $form_values['group_name'], $form_values['weight'], $form_values['rid']);
		drupal_set_message(t('Changes to the rule saved.'));
	} else {
		db_query('insert into {hellomobile_rules} (rid,capability,operator,value,group_name,weight) values (%d,"%s","%s","%s","%s",%d)',db_next_id('{hellomobile_rules}_rid'), $form_values['capability'], $form_values['operator'], $form_values['value'], $form_values['group_name'], $form_values['weight']);
		drupal_set_message(t('New rule added.'));
	}
	cache_clear_all('hellomobile_rulelist', 'cache');
	return 'admin/build/hellomobile/rules';
}

/**
 * Starts a test using the given group.
 * @param group a group name to test
 */
function hellomobile_test_group($group = false) {
	global $hellomobile_group;
	if (!$group) {
		unset($_SESSION['hellomobile_group']);
	} else if ($group == '_touchscreen') {
		$_SESSION['hellomobile_group'] = $hellomobile_group . ',1';
	} else {
		$_SESSION['hellomobile_group'] = _hellomobile_validate_group($group) . ',0';
	}
	drupal_goto('admin/build/hellomobile/groups');
}

/**
 * Flushes cached node optimizations and scaled images.
 */
function _hellomobile_flush() {
	cache_clear_all('hellomobile_', 'cache', true);
	cache_clear_all('hellomobile_', 'cache_page', true);
	_hellomobile_rmdir(file_directory_path() . '/hellomobile');
	drupal_set_message(t('Hello Mobile caches cleared.'));
	drupal_goto('admin/build/hellomobile');
}

/**
 * Removes a directory with it's contents.
 * @param dirname a directory path to remove
 * @return true on success
 */
function _hellomobile_rmdir($dirname) {
	if (!file_exists($dirname)) {
		return false;
	}
	if (is_file($dirname) || is_link($dirname)) {
		return unlink($dirname);
	}
	$dir = dir($dirname);
	while (false !== $entry = $dir->read()) {
		if ($entry == '.' || $entry == '..') {
			continue;
		}
		_hellomobile_rmdir($dirname . '/' . $entry);
	}
	$dir->close();
	return rmdir($dirname);
}
